#!/usr/bin/env python
# coding: utf-8

from __future__ import print_function

import io
import re
import sys
from builtins import *

import xmltodict

import marcx
from siskin.utils import marc_build_imprint

lang_map = {
    "deutsch": "ger",
    "englisch": "eng",
    "franzoesisch": "fre",
    "spanisch": "spa",
    "portugiesisch": "por",
    "italienisch": "ita",
    "tuerkisch": "tur",
    "schwedisch": "swe"
}

# Default input and output.
inputfilename = "78_input.xml"
outputfilename = "78_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:]

inputfile = io.open(inputfilename, "r", encoding='utf-8')
outputfile = io.open(outputfilename, "wb")
xmlfile = inputfile.read()
xmlrecords = xmltodict.parse(xmlfile)

for xmlrecord in xmlrecords["IZI_Datensaetze"]["Datensatz"]:

    marcrecord = marcx.Record(force_utf8=True)
    marcrecord.strict = False

    # Formaterkennung
    format = xmlrecord["PUBLIKATIONSTYP"]
    nummer = xmlrecord["NUMMER"]
    url = xmlrecord["URL"]
    isbn = xmlrecord["ISBN"]
    zeitschrift = xmlrecord["ZEITSCHRIFT"]
    #beitrag = xmlrecord["SAMMELWERK"]
    band = xmlrecord["BAND"]
    quelle = xmlrecord["QUELLE"]

    if quelle:
        match = re.search(".*\sS\.\s\d+-\d+", quelle)
        if match:
            artikel = True
        else:
            artikel = False

    if not format:
        format = ""

    if ("Bachelorarbeit" in format or "Masterarbeit" in format or "Diplomarbeitarbeit" in format
            or "Magisterarbeitarbeit" in format or "Dissertation" in format):

        leader = "     nam  22        4500"
        f935c = "hs"

    elif isbn:
        leader = "     nam  22        4500"
        f935c = "lo"

    elif zeitschrift or artikel or "beitrag" in format:
        leader = "     naa  22        4500"
        f935c = "text"

    elif quelle:
        leader = "     nam  22        4500"
        f935c = "lo"

    else:
        leader = "     nam  22        4500"
        f935c = "lo"

    if url:
        f007 = "cr"
        f935b = "cofz"
    else:
        f007 = "tu"
        f935b = "druck"

    # Leader
    marcrecord.leader = leader

    # Identifier
    f001 = xmlrecord["NUMMER"]
    if not f001:
        continue
    marcrecord.add("001", data="finc-78-" + f001)

    # 007
    marcrecord.add("007", data=f007)

    # ISBN
    isbns = xmlrecord["ISBN"]
    if isbns:
        isbns = isbns.split("; ")
        for f020a in isbns:
            marcrecord.add("020", a=f020a)

    # Sprache
    languages = xmlrecord["SPRACHE"]
    if languages:
        languages = languages.split("; ")
        language = languages[0]
        language = lang_map.get(language, "")
        if language != "":
            marcrecord.add("008", data="130227uu20uuuuuuxx uuu %s  c" % language)
            if len(languages) < 4:
                subfields = []
                for f041a in languages:
                    subfields.append("a")
                    f041a = lang_map.get(f041a, "")
                    subfields.append(f041a)
                    marcrecord.add("041", subfields=subfields)
            else:
                marcrecord.add("041", a="mul")
        else:
            print("Die Sprache %s fehlt in der Lang_Map!" % language)

    # 1. Urheber
    persons = xmlrecord["AUTOR"]
    if persons:
        persons = persons.split("; ")
        f100a = persons[0]
        marcrecord.add("100", a=f100a)
    else:
        persons = []

    # körperschaftlicher Urheber
    corporates = xmlrecord["KORP_URHEBER"]
    if corporates:
        corporates = corporates.split("; ")
        f110a = corporates[0]
        marcrecord.add("110", a=f110a)

    # Haupttitel
    f245a = xmlrecord["TITEL"]
    f245b = xmlrecord["BAND"]
    if f245b:
        marcrecord.add("245", a=f245a, b=f245b)
    else:
        marcrecord.add("245", a=f245a)

    # Erscheinungsvermerk
    year = xmlrecord["JAHR"]
    f260 = xmlrecord["QUELLE"]
    if f260:
        match = re.search("(.*?):\s(.*?)\s(\d\d\d\d)[,\.]\s.*", f260)
        if match:
            f260a, f260b, f260c = match.groups()
        else:
            f260a = ""
            f260b = ""
            f260c = ""

    if year:
        f260c = year

    subfields = marc_build_imprint(f260a, f260b, f260c)
    marcrecord.add("260", subfields=subfields)

    # Umfangsangabe
    f300a = xmlrecord["QUELLE"]
    if f300a:
        match = re.search(".*\s\d\d\d\d[,\.]\s(.*)", f300a)
        if match:
            f300a = match.groups()
            marcrecord.add("300", a=f300a)

    # Reihentitel
    f490a = xmlrecord["REIHENTITEL"]
    if f490a:
        f490a = f490a.replace("(", "")
        f490a = f490a.replace(")", "")
        f490a = re.sub("\.\s(\d)", r" ; \1", f490a)
        marcrecord.add("490", a=f490a)

    # Fußnote
    languages = xmlrecord["SPRACHE"]
    if languages:
        numlang = languages.split()
        if len(numlang) > 3:
            languages = languages.replace(";", ",")
            f500a = "Text auf " + languages.title()
            marcrecord.add("500", a=f500a)

    # Schlagwort
    subjects = xmlrecord["SCHLAGWORT"]
    subjects = subjects.split("; ")
    for f650a in subjects:
        marcrecord.add("650", a=f650a)

    # weitere Urheber
    for f700a in persons[1:]:
        marcrecord.add("700", a=f700a)

    editors = xmlrecord["HRSG_MITARBEITER"]
    if editors:
        editors = editors.split("; ")
        for f700a in editors:
            marcrecord.add("700", a=f700a)

    # weitere körperschaftliche Urheber
    if corporates:
        for f710a in corporates[1:]:
            if f710a:
                f710a = f710a.replace(" (Hrsg.)", "")
                marcrecord.add("710", a=f710a)

    # körperschaftliche Herausgeber
    corporate_editors = xmlrecord["KORP_HRSG"]
    if corporate_editors:
        corporate_editors = corporate_editors.split("; ")
        for f710a in corporate_editors[1:]:
            if f710a:
                f710a = f710a.replace(" (Hrsg.)", "")
                marcrecord.add("710", a=f710a)

    # übergeordnete Ressource
    f773g = xmlrecord["ZEITSCHRIFT"]
    marcrecord.add("773", g=f773g)

    f773t = xmlrecord["SAMMELWERK"]
    marcrecord.add("773", t=f773t)

    # Link zur Ressource
    f856u = xmlrecord["URL"]
    if f856u:
        marcrecord.add("856", q="text/html", _3="Link zur Ressource", u=f856u)

    # Link zum Datensatz
    f856u = "http://www.izi-datenbank.de/details/" + f001
    marcrecord.add("856", q="text/html", _3="Link zum Datensatz", u=f856u)

    # Medienform
    marcrecord.add("935", b=f935b, c=f935c)

    # Kollektion
    marcrecord.add("980", a=f001, b="78", c="sid-78-col-izi")

    outputfile.write(marcrecord.as_marc())

inputfile.close()
outputfile.close()



#!/usr/bin/env python
# coding: utf-8
# pylint: disable=C0103
"""
Note: This is WIP. Currently Python 2 only.

Input file is an OAI DC XML, processing currently tailored for SID 103.

    $ python 103.py INFILE OUTFILE

"""

from __future__ import print_function

import base64
import cgi
import io
import os
import re
import sqlite3
import sys
import tempfile

import xmltodict

if len(sys.argv) < 3:
    print('usage: %s INFILE OUTFILE' % sys.argv[0], file=sys.stderr)
    sys.exit(1)

_, dbfile = tempfile.mkstemp(prefix='103-')

sqlitecon = sqlite3.connect(dbfile)
sqlite = sqlitecon.cursor()
sqlite.execute("DROP TABLE IF EXISTS record")

query = """
    CREATE TABLE
       'record' ('id' INTEGER PRIMARY KEY AUTOINCREMENT,
                 'identifier' TEXT,
                 'title' TEXT,
                 'description' TEXT,
                 'subject' TEXT,
                 'type' TEXT,
                 'format' TEXT,
                 'relation' TEXT,
                 'publisher' TEXT,
                 'date' TEXT,
                 'source' TEXT,
                 'language' TEXT,
                 'rights' TEXT,
                 'url' TEXT,
                 'creator' TEXT,
                 'coverage' TEXT)
"""

sqlite.execute(query)

inputfile = io.open(sys.argv[1], "r", encoding='utf-8')
outputfile = io.open(sys.argv[2], "w", encoding='utf-8')

xml = xmltodict.parse(inputfile.read())

identifier = ""
title = ""
description = ""
subject = ""
type = ""
format = ""
relation = ""
publisher = ""
date = ""
source = ""
language = ""
rights = ""
url = ""
creator = ""
coverage = ""

inserts = 0
updates = 0

for i, record in enumerate(xml["collection"]["Record"]):

    if record["header"]["@status"] != "deleted":

        identifier = record["header"]["identifier"]

        try:
            title = record["metadata"]["oai_dc:dc"]["dc:title"]
            if isinstance(title, list):
                title = '||'.join(record["metadata"]["oai_dc:dc"]["dc:title"])
        except (TypeError, KeyError):
            title = ''

        try:
            description = record["metadata"]["oai_dc:dc"]["dc:description"]
            if isinstance(description, list):
                description = '||'.join(record["metadata"]["oai_dc:dc"]["dc:description"])
        except (TypeError, KeyError):
            description = ''

        try:
            subject = record["metadata"]["oai_dc:dc"]["dc:subject"]
            if isinstance(subject, list):
                subject = '; '.join(record["metadata"]["oai_dc:dc"]["dc:subject"])
        except (TypeError, KeyError):
            subject = ''

        try:
            type = record["metadata"]["oai_dc:dc"]["dc:type"]
            if isinstance(type, list):
                type = '||'.join(record["metadata"]["oai_dc:dc"]["dc:type"])
        except (TypeError, KeyError):
            type = ''

        try:
            format = record["metadata"]["oai_dc:dc"]["dc:format"]
            if isinstance(format, list):
                format = '||'.join(record["metadata"]["oai_dc:dc"]["dc:format"])
        except (TypeError, KeyError):
            format = ''

        try:
            relation = record["metadata"]["oai_dc:dc"]["dc:relation"]
            if isinstance(relation, list):
                relation = '||'.join(record["metadata"]["oai_dc:dc"]["dc:relation"])
        except (TypeError, KeyError):
            relation = ''

        try:
            publisher = record["metadata"]["oai_dc:dc"]["dc:publisher"]
            if isinstance(publisher, list):
                publisher = '||'.join(record["metadata"]["oai_dc:dc"]["dc:publisher"])
        except (TypeError, KeyError):
            publisher = ''

        try:
            date = record["metadata"]["oai_dc:dc"]["dc:date"]
            if isinstance(date, list):
                date = '||'.join(record["metadata"]["oai_dc:dc"]["dc:date"])
        except (TypeError, KeyError):
            date = ''

        try:
            source = record["metadata"]["oai_dc:dc"]["dc:source"]
            if isinstance(source, list):
                source = record["metadata"]["oai_dc:dc"]["dc:source"][0]
        except (TypeError, KeyError):
            source = ''

        try:
            language = record["metadata"]["oai_dc:dc"]["dc:language"]
            if isinstance(language, list):
                language = '||'.join(record["metadata"]["oai_dc:dc"]["dc:language"])
        except (TypeError, KeyError):
            language = ''

        try:
            rights = record["metadata"]["oai_dc:dc"]["dc:rights"]
            if isinstance(rights, list):
                rights = '||'.join(record["metadata"]["oai_dc:dc"]["dc:rights"])
        except (TypeError, KeyError):
            rights = ''

        try:
            url = record["metadata"]["oai_dc:dc"]["dc:identifier"]
            if isinstance(url, list):
                url = record["metadata"]["oai_dc:dc"]["dc:identifier"][1]
        except (TypeError, KeyError):
            url = ''

        try:
            creator = record["metadata"]["oai_dc:dc"]["dc:creator"]
            if isinstance(creator, list):
                creator = '||'.join(record["metadata"]["oai_dc:dc"]["dc:creator"])
        except (TypeError, KeyError):
            creator = ''

        try:
            coverage = record["metadata"]["oai_dc:dc"]["dc:coverage"]
            if isinstance(coverage, list):
                coverage = '||'.join(record["metadata"]["oai_dc:dc"]["dc:coverage"])
        except (TypeError, KeyError):
            coverage = ''

        values = (title, format, type)

        query = """
            SELECT
                id,
                url,
                COUNT(id) AS matches
            FROM
                record
            WHERE
                title = ? AND
                format = ? AND
                type = ?
        """

        sqlite.execute(query, values)
        row = sqlite.fetchone()

        id = row[0]
        url_old = row[1]
        matches = row[2]

        if matches == 0:

            values = (identifier, title, description, subject, type, format, relation, publisher, date, source,
                      language, rights, url, creator, coverage)

            query = """
                INSERT INTO
                    record (identifier,
                            title,
                            description,
                            subject,
                            type,
                            format,
                            relation,
                            publisher,
                            date,
                            source,
                            language,
                            rights,
                            url,
                            creator,
                            coverage)
                VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
            """

            inserts = inserts + 1

        else:

            url = url_old + "||" + url
            values = (url, id)

            query = """
                UPDATE
                    record
                SET
                    url = ?
                WHERE
                    id = ?
            """

            updates = updates + 1

        sqlite.execute(query, values)

        identifier = ""
        title = ""
        description = ""
        subject = ""
        type = ""
        format = ""
        relation = ""
        publisher = ""
        date = ""
        source = ""
        language = ""
        rights = ""
        url = ""
        creator = ""
        coverage = ""

        #print("Inserts:", inserts, "Updates", updates)

query = "SELECT * FROM record"
sqlite.execute(query)
rows = sqlite.fetchall()

outputfile.write(u"<collection>")
outputfile.write(u"\n")

for row in rows:
    identifier = row[1]
    identifier = base64.b64encode(identifier.encode()).decode().rstrip("=")
    title = row[2]
    title = cgi.escape(title)
    description = row[3]
    description = cgi.escape(description)
    subject = row[4]
    subject = cgi.escape(subject)
    type = row[5]
    format = row[6]
    relation = row[7]
    publisher = row[8]
    publisher = cgi.escape(publisher)
    date = row[9]
    source = row[10]
    source = cgi.escape(source)
    language = row[11]
    rights = row[12]
    rights = cgi.escape(rights)
    url = row[13]
    creator = row[14]
    creator = cgi.escape(creator)
    coverage = row[15]
    coverage = cgi.escape(coverage)

    if identifier == "":
        continue

    # Zur Gewährleistung der korrekten Reihenfolge der Tags werden zunächst alle Variablen deklariert und erst dann die Datei geschrieben

    f001 = "\t\t<controlfield tag=\"001\">finc-103-%s</controlfield>\n" % identifier

    # 2017-06-16: Example rewrite of some XML:
    #
    # formatmaps = {
    #     'periodical': {
    #         '007': 'tu',
    #         '008': '130227n20uuuuuuxx uuupger c',
    #         'leader': 'nas',
    #         '935b' : 'cofz',
    #     },
    #     'ephemera': {'007': 'ta', 'leader': 'ckm', '935b' : 'foto'},
    #     'magazine cover': {'007': 'ta', 'leader': 'ckm', '935b' : 'foto'},
    #     'photograph': {'007': 'ta', 'leader': 'ckm', '935b' : 'foto'},
    #     'postcard': {'007': 'ta', 'leader': 'ckm', '935b' : 'foto'},
    #     'clipping': {'007': 'ta', 'leader': 'ckm', '935b' : 'foto'},
    #     'flier (printed matter)': {'007': 'kf', 'leader': 'nam'},
    #     'storyboard': {'007': 'kf', 'leader': 'nam'},
    #     'sheet music': {'007': 'qu', 'leader': 'ncs'},
    #     'correspondence': {'007': 'kf', 'leader': 'nam'},
    #     'pamphlet': {'007': 'kf', 'leader': 'nam'},
    #     'correspondence document': {'007': 'kf', 'leader': 'nam'},
    #     'lobby card': {'007': 'ta', 'leader': 'ckm', '935b' : 'foto'},
    #     'photomechanical print': {'007': 'kf', 'leader': 'nam'},
    #     'cigarette cards': {'007': 'kd', 'leader': 'nam'},
    #     'souvenir handkerchief box': {'007': 'ta', 'leader': 'ckm', '935b' : 'foto'},
    #     'scrapbook': {'007': 'cr', 'leader': 'nam', '935b' : 'cofz'},
    #     'slides (photographs)': {'007': 'ta', 'leader': 'ckm', '935b' : 'foto'},
    #     'program (document)': {'007': 'cr', 'leader': 'nam', '935b' : 'cofz'},
    #     'pressbook': {'007': 'cr', 'leader': 'nam', '935b' : 'cofz'},
    #     'autograph album': {'007': 'tu', 'leader': 'ntm', '935b' : 'handschr'},
    #     'monograph': {'007': 'cr', 'leader': 'nam', '935b' : 'cofz'},
    #     'drawing': {'007': 'kd', 'leader': 'nam'},
    # }

    # TODO:
    # format not found: pamphlet;
    # format not found: correspondence; document
    # format not found: flier (printed matter);
    # format not found: Monograph
    # format not found: ephemera;
    # format not found: Periodical
    # format not found:
    # format not found: document
    # if format not in formatmaps:
    #     print("format not found: %s" % format, file=sys.stderr)
    #     continue

    # def get_field_007(format='photograph'):
    #     """ Return XML for field 007 given a format. """
    #     return """<controlfield tag="007">%s</controlfield>""" % formatmaps[format]["007"]

    # def get_field_008(format='photograph'):
    #     if "008" not in formatmaps[format]:
    #         return ""
    #     return """
    #         <controlfield tag="008">%s</controlfield>
    #     """ % formatmaps[format]["008"]

    # def get_leader(format='photograph'):
    #     """ Return XML for leader given a format. """
    #     return "<leader>00000%s0000000000000000</leader>" % formatmaps[format]["leader"]

    # def get_field_935b(format='photograph'):
    #     """ Return XML for 935.b given a format. """
    #     if "935b" not in formatmaps[format]:
    #         return ""
    #     return """
    #         <datafield tag="935" ind1=" " ind2=" ">
    #             <subfield code="b">%s</subfield>
    #         </datafield>
    #     """ % formatmaps[format]["935b"]

    # leader = get_leader(format=format)
    # f007 = get_field_007(format=format)
    # f008 = get_field_008(format=format)
    # f935b = get_field_935b(format=format)

    # ElectronicJournal
    if format == "periodical":
        leader = "\t\t<leader>00000nas0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">tu</controlfield>\n"
        f008 = "\t\t<controlfield tag=\"008\">130227n20uuuuuuxx uuupger c</controlfield>\n"
        f935b = "\t\t<datafield tag=\"935\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"b\">cofz</subfield>\n\t\t</datafield>\n"

    # Photo (Image gibt es nicht)
    elif format == "ephemera":
        leader = "\t\t<leader>00000ckm0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">ta</controlfield>\n"
        f008 = ""
        f935b = "\t\t<datafield tag=\"935\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"b\">foto</subfield>\n\t\t</datafield>\n"

    # Photo
    elif format == "magazine cover":
        leader = "\t\t<leader>00000ckm0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">ta</controlfield>\n"
        f008 = ""
        f935b = "\t\t<datafield tag=\"935\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"b\">foto</subfield>\n\t\t</datafield>\n"

    # Photo
    elif format == "photograph":
        leader = "\t\t<leader>00000ckm0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">ta</controlfield>\n"
        f008 = ""
        f935b = "\t\t<datafield tag=\"935\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"b\">foto</subfield>\n\t\t</datafield>\n"

    # Photo (Image gibt es nicht)
    elif format == "postcard":
        leader = "\t\t<leader>00000ckm0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">ta</controlfield>\n"
        f008 = ""
        f935b = "\t\t<datafield tag=\"935\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"b\">foto</subfield>\n\t\t</datafield>\n"

    # Photo
    elif format == "clipping":
        leader = "\t\t<leader>00000ckm0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">ta</controlfield>\n"
        f008 = ""
        f935b = "\t\t<datafield tag=\"935\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"b\">foto</subfield>\n\t\t</datafield>\n"

    # Print
    elif format == "flier (printed matter)":
        leader = "\t\t<leader>00000nam0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">kf</controlfield>\n"
        f008 = ""
        f935b = ""

    # Print
    elif format == "storyboard":
        leader = "\t\t<leader>00000nam0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">kf</controlfield>\n"
        f008 = ""
        f935b = ""

    # MusicalScore (ElectronicMusicalScore gibt es nicht)
    elif format == "sheet music":
        leader = "\t\t<leader>00000ncs0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">qu</controlfield>\n"
        f008 = ""
        f935b = ""

    # Print
    elif format == "correspondence":
        leader = "\t\t<leader>00000nam0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">kf</controlfield>\n"
        f008 = ""
        f935b = ""

    # Print
    elif format == "pamphlet":
        leader = "\t\t<leader>00000nam0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">kf</controlfield>\n"
        f008 = ""
        f935b = ""

    # Print
    elif format == "correspondence document":
        leader = "\t\t<leader>00000nam0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">kf</controlfield>\n"
        f008 = ""
        f935b = ""

    # Photo
    elif format == "lobby card":
        leader = "\t\t<leader>00000ckm0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">ta</controlfield>\n"
        f008 = ""
        f935b = "\t\t<datafield tag=\"935\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"b\">foto</subfield>\n\t\t</datafield>\n"

    # Print
    elif format == "photomechanical print":
        leader = "\t\t<leader>00000nam0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">kf</controlfield>\n"
        f008 = ""
        f935b = ""

    # Drawing
    elif format == "cigarette cards":
        leader = "\t\t<leader>00000nam0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">kd</controlfield>\n"
        f008 = ""
        f935b = ""

    # Photo
    elif format == "souvenir handkerchief box":
        leader = "\t\t<leader>00000ckm0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">ta</controlfield>\n"
        f008 = ""
        f935b = "\t\t<datafield tag=\"935\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"b\">foto</subfield>\n\t\t</datafield>\n"

    # ElectronicBook
    elif format == "scrapbook":
        leader = "\t\t<leader>00000nam0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">cr</controlfield>\n"
        f008 = ""
        f935b = "\t\t<datafield tag=\"935\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"b\">cofz</subfield>\n\t\t</datafield>\n"

    # Photo
    elif format == "slides (photographs)":
        leader = "\t\t<leader>00000ckm0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">ta</controlfield>\n"
        f008 = ""
        f935b = "\t\t<datafield tag=\"935\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"b\">foto</subfield>\n\t\t</datafield>\n"

    # ElectronicBook
    elif format == "program (document)":
        leader = "\t\t<leader>00000nam0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">cr</controlfield>\n"
        f008 = ""
        f935b = "\t\t<datafield tag=\"935\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"b\">cofz</subfield>\n\t\t</datafield>\n"

    # ElectronicBook
    elif format == "pressbook":
        leader = "\t\t<leader>00000nam0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">cr</controlfield>\n"
        f008 = ""
        f935b = "\t\t<datafield tag=\"935\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"b\">cofz</subfield>\n\t\t</datafield>\n"

    # ElectronicManuscript
    elif format == "autograph album":
        leader = "\t\t<leader>00000ntm0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">tu</controlfield>\n"
        f008 = "\t\t<controlfield tag=\"008\">130227n20uuuuuuxx uuumger c</controlfield>\n"
        f935b = "\t\t<datafield tag=\"935\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"b\">handschr</subfield>\n\t\t</datafield>\n"

    # ElectronicBook
    elif format == "monograph":
        leader = "\t\t<leader>00000nam0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">cr</controlfield>\n"
        f008 = ""
        f935b = "\t\t<datafield tag=\"935\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"b\">cofz</subfield>\n\t\t</datafield>\n"

    # Drawing
    elif format == "drawing":
        leader = "\t\t<leader>00000nam0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">kd</controlfield>\n"
        f008 = ""
        f935b = ""

    # Photo
    else:
        leader = "\t\t<leader>00000ckm0000000000000000</leader>\n"
        f007 = "\t\t<controlfield tag=\"007\">ta</controlfield>\n"
        f008 = ""
        f935b = "\t\t<datafield tag=\"935\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"b\">foto</subfield>\n\t\t</datafield>\n"

    if language != "":
        f041 = language.replace("English", "eng")
        f041 = language.replace("German", "ger")
        f041 = language.replace("French", "fre")
        f041 = language.replace("Italian", "ita")
        f041 = "\t\t<datafield tag=\"041\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"a\">" + str(
            f041) + "</subfield>\n\t\t</datafield>\n"
    else:
        f041 = ""

    if creator != "":
        creator = creator.split(" ; ")
        f100 = creator[0]
        f100 = "\t\t<datafield tag=\"100\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"a\">" + str(
            f100) + "</subfield>\n\t\t</datafield>\n"
        if len(creator) > 1:
            f700 = []
            for creator_part in creator[1:]:
                creator_part = "\t\t<datafield tag=\"700\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"a\">" + \
                    str(creator_part) + "</subfield>\n\t\t</datafield>\n"
                f700.append(creator_part)
            f700 = "".join(f700)
        else:
            f700 = ""
    else:
        f100 = ""
        f700 = ""

    if title != "":
        f245 = "\t\t<datafield tag=\"245\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"a\">" + unicode(
            title) + "</subfield>\n\t\t</datafield>\n"
    else:
        f245 = ""

    # Chicago : Selig Polyscope Co.
    # Shurey's Publications
    if publisher != "":
        match = re.search("(.*)\s:\s(.*)", publisher)
        if match:
            place = match.group(1)
            place = "\n\t\t\t<subfield code=\"a\">" + str(place) + "</subfield>"
            publisher = match.group(2)
            publisher = "\n\t\t\t<subfield code=\"b\"> : " + unicode(publisher) + "</subfield>"
        else:
            place = ""
            publisher = "\n\t\t\t<subfield code=\"b\">" + unicode(publisher) + "</subfield>"
    else:
        publisher = ""
        place = ""

    # 1932-04-01
    #1930; 1931
    # 1922
    if date != "":
        match = re.search("(^\d\d\d\d)", date)
        if match:
            date = match.group(1)
            if publisher != "":
                date = "\n\t\t\t<subfield code=\"c\">, " + str(date) + "</subfield>"
            else:
                date = "\n\t\t\t<subfield code=\"c\">" + str(date) + "</subfield>"
        else:
            date = ""
    else:
        date = ""

    if place != "" or publisher != "" or date != "":
        f260 = "\t\t<datafield tag=\"260\" ind1=\" \" ind2=\" \">" + unicode(place) + unicode(publisher) + unicode(
            date) + "\n\t\t</datafield>\n"
    else:
        f260 = ""

    if source != "":
        f490 = "\t\t<datafield tag=\"490\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"a\">" + str(
            source) + "</subfield>\n\t\t</datafield>\n"
    else:
        f490 = ""

    if rights != "" and coverage != "":
        f500 = []
        f500.append("\t\t<datafield tag=\"500\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"a\">" + str(rights) +
                    "</subfield>\n\t\t</datafield>\n")
        f500.append("\t\t<datafield tag=\"500\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"a\">" + str(coverage) +
                    "</subfield>\n\t\t</datafield>\n")
        f500 = "".join(f500)
    elif rights != "":
        f500 = "\t\t<datafield tag=\"500\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"a\">" + str(
            rights) + "</subfield>\n\t\t</datafield>\n"
    elif coverage != "":
        f500 = "\t\t<datafield tag=\"500\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"a\">" + str(
            coverage) + "</subfield>\n\t\t</datafield>\n"
    else:
        f500 = ""

    if description != "":
        description = description.split("||")
        f520 = []
        for description_part in description:
            f520.append("\t\t<datafield tag=\"520\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"a\">" +
                        unicode(description_part) + "</subfield>\n\t\t</datafield>\n")
        f520 = "".join(f520)
    else:
        f520 = ""

    if subject != "" or format != "":
        # manchmal ended die Schlagwortfolgfe auf ;
        subject = subject.rstrip(";")
        # die Trennzeichen sind in der Quelle uneinheitlich: mal " ; " mal "; "
        # Problem: Semikolon ist Trennzeichen zwischen Schlagwörtern, kommt aber auch als Funktionszeichen in HTML-Entitities (&amp;) vor
        match = re.search("&\w\w\w?\w?;", subject)

        # wenn keine HTMl-Entities vorkommen, werden die Trennzeichen vereinheitlicht
        if not match:
            subject = subject.replace(" ; ", ";")
            subject = subject.replace(" ;", ";")
            subject = subject.replace("; ", ";")
            subject = subject.split(";")
        # wenn HTMl-Entities vorkommen, wird nur das reguläre Trennzeichen verwendet und abweichende Fälle bleiben unberücksichtigt
        else:
            subject = subject.split(" ; ")

        f689 = []
        for subject_part in subject:
            f689.append("\t\t<datafield tag=\"689\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"a\">" +
                        unicode(subject_part) + "</subfield>\n\t\t</datafield>\n")

        format = format.title()
        f689.append("\t\t<datafield tag=\"689\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"a\">" + str(format) +
                    "</subfield>\n\t\t</datafield>\n")
        f689 = "".join(f689)

    else:
        f689 = ""

    # if relation != "":
    #   outputfile.write(u"\t\t<datafield tag=\"856\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"q\">text/html</subfield>\n\t\t\t<subfield code=\"3\">Link zur Reihe</subfield>\n\t\t\t<subfield code=\"u\">" + str(relation) + "</subfield>\n\t\t</datafield>\n")

    if url != "":
        url = url.split("||")
        if len(url) == 1:
            url = url[0]
            f856 = "\t\t<datafield tag=\"856\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"q\">text/html</subfield>\n\t\t\t<subfield code=\"3\">Link zur Ressource</subfield>\n\t\t\t<subfield code=\"u\">" + \
                str(url) + "</subfield>\n\t\t</datafield>\n"
        else:
            items = len(url)
            f520 = "\t\t<datafield tag=\"520\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"a\">Contains a collection of " + \
                str(items) + " Pictures.</subfield>\n\t\t</datafield>\n"
            f856 = []
            i = 1
            for url_part in url:
                f856.append(
                    "\t\t<datafield tag=\"856\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"q\">text/html</subfield>\n\t\t\t<subfield code=\"3\">Picture "
                    + str(i) + "</subfield>\n\t\t\t<subfield code=\"u\">" + str(url_part) +
                    "</subfield>\n\t\t</datafield>\n")
                i = i + 1
            f856 = "".join(f856)
    else:
        f856 = ""

    f980 = "\t\t<datafield tag=\"980\" ind1=\" \" ind2=\" \">\n\t\t\t<subfield code=\"a\">%s</subfield>\n\t\t\t<subfield code=\"b\">103</subfield>\n\t\t</datafield>\n" % identifier

    outputfile.write(u"\t<record>\n")
    outputfile.write(unicode(leader))
    outputfile.write(unicode(f001))
    outputfile.write(unicode(f007))
    outputfile.write(unicode(f008))
    outputfile.write(unicode(f041))
    outputfile.write(unicode(f100))
    outputfile.write(unicode(f245))
    outputfile.write(unicode(f260))
    outputfile.write(unicode(f490))
    outputfile.write(unicode(f500))
    outputfile.write(unicode(f520))
    outputfile.write(unicode(f689))
    outputfile.write(unicode(f700))
    outputfile.write(unicode(f856))
    outputfile.write(unicode(f935b))
    outputfile.write(unicode(f980))
    outputfile.write(u"\t</record>\n")

outputfile.write(u"</collection>")

sqlitecon.commit()
sqlitecon.close()
inputfile.close()
outputfile.close()
os.remove(dbfile)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301
# Copyright 2015 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Tracy Hoffmann, <tracy.hoffmann@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
IZI Task #7755, #13652, #14692.

Config:

[izi]

input = /path/to/izi.xml

"""

import datetime
import os

import luigi

from gluish.format import Gzip
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask


class IZITask(DefaultTask):
    """ Base task for IZI. """
    TAG = '78'


class IZIMARC(IZITask):
    """
    Convert to binary MARC.
    """

    def run(self):
        output = shellout("""python {script} {input} {output}""",
                          script=self.assets('78/78_marcbinary.py'),
                          input=self.config.get('izi', 'input'))
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='fincmarc.mrc'))



# /usr/bin/env python3
# coding: utf-8

import re
import sys

import marcx
import pymarc

copytags = ("003", "005", "006", "007", "008", "020", "022", "024", "035", "040", "084", "100", "110", "245", "246",
            "260", "300", "310", "362", "490", "520", "650", "651", "700", "710", "760", "762", "773", "775", "780",
            "785", "830")

inputfilename = "52_input.xml"
outputfilename = "52_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:]

inputfile = open(inputfilename, "rb")
outputfile = open(outputfilename, "wb")

# reader = pymarc.MARCReader(inputfile, force_utf8=True)
with open(inputfilename) as handle:
    records = pymarc.parse_xml_to_array(handle)

for oldrecord in records:

    newrecord = marcx.Record(force_utf8=True)

    # leader
    leader = "     " + oldrecord.leader[5:]
    newrecord.leader = leader

    # 001
    f001 = oldrecord["001"].data
    f001 = f001.replace("-", "")
    f001 = f001.replace("_", "")
    newrecord.add("001", data="finc-52-%s" % f001)

    # ISBN
    try:
        f020a = oldrecord["020"]["a"]
    except:
        f020a = ""

    if f020a != "":
        f020a = f020a.replace(" ", "-")
        f020a = f020a.replace(".", "-")
        regexp = re.search("([0-9xX-]{10,17})", f020a)

        if regexp:
            f020a = regexp.group(1)
            f020a = f020a.rstrip("-")
            newrecord.add("020", a=f020a)
        else:
            print("Die ISBN %s konnte nicht mittels regulärer Ausdrücke überprüft werden." % f020a)

    # Originalfelder, die ohne Änderung übernommen werden
    for tag in copytags:
        for field in oldrecord.get_fields(tag):
            newrecord.add_field(field)

    # Inhaltsverzeichnis (muss trunkiert werden auf maximal 9999 Bytes)
    try:
        f505a = oldrecord["505"]["a"]
        if len(f505a) > 9800:
            f505a = f505a[:9799]
        newrecord.add("505", a=f505a)
    except (AttributeError, TypeError) as err:
        pass

    # Schlagwort
    try:
        f689a = oldrecord["653"]["a"]
        newrecord.add("689", a=f689a)
    except (AttributeError, TypeError):
        pass

    # URL
    f856u = oldrecord["856"]["u"]
    newrecord.add("856", q="text/html", _3="Link zur Ressource", u=f856u)

    # 980
    collections = ["a", f001, "b", "52", "c", "sid-52-col-oecd"]
    newrecord.add("980", subfields=collections)

    outputfile.write(newrecord.as_marc())

inputfile.close()
outputfile.close()



# coding: utf-8
# pylint: disable=C0301,R0904,W0221

# Copyright 2015 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
Configuration handling, taken from luigi - ini format.
"""

try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import SafeConfigParser as ConfigParser

import datetime
import logging
import os
import sys

logger = logging.getLogger('siskin')


class Config(ConfigParser):
    """
    Access to ini file.
    """
    NO_DEFAULT = None
    _instance = None

    # most specific path last
    _config_paths = [
        '/etc/siskin/siskin.ini',
        os.path.join(os.path.expanduser('~'), '.config/siskin/siskin.ini'),
    ]

    @classmethod
    def add_config_path(cls, path):
        """ Append config path. """
        cls._config_paths.append(path)
        cls._instance.reload()

    @classmethod
    def instance(cls, *args, **kwargs):
        """ Singleton getter """
        if cls._instance is None:
            cls._instance = cls(*args, **kwargs)
            _ = cls._instance.reload()

        return cls._instance

    def reload(self):
        """ Reload configuration. """
        return self._instance.read(self._config_paths)



# coding: utf-8
#
# Copyright 2018 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>#
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
GDMB Gesellschaft der Metallurgen und Bergleute, refs #9950.

Download data manually from https://ownsky.hrz.tu-freiberg.de/owncloud/index.php/s/NCRbjTu3ab0oLsj

Configuration:

[gdmb]

input = /path/to/json (ask RS)
"""

import luigi

from gluish.utils import shellout
from siskin.task import DefaultTask


class GDMBTask(DefaultTask):
    """
    This task inherits functionality from `siskin.task.DefaultTask`.
    """
    TAG = '131'


class GDMBMARC(GDMBTask):
    """ Convert JSON to BinaryFincMarc """

    def run(self):
        output = shellout("""python {script} {input} {output}""",
                          script=self.assets("131/131_marcbinary.py"),
                          input=self.config.get("gdmb", "input"))
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='fincmarc.mrc'))



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301

# Copyright 2018 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
Geoscan, refs #9871.

* http://geoscan.nrcan.gc.ca/rss/newpub_e.rss

XXX: Only recent works?
"""

import datetime

import luigi

from gluish.intervals import weekly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask


class GeoscanTask(DefaultTask):
    """
    Base task for Geoscan.
    """
    TAG = "129"

    def closest(self):
        return weekly(date=self.date)


class GeoscanMARC(GeoscanTask):
    """
    Geoscan RSS to MARC. Currently, only the current feed is converted.
    """
    date = ClosestDateParameter(default=datetime.date.today())
    link = luigi.Parameter(default="http://geoscan.nrcan.gc.ca/rss/newpub_e.rss", significant=False)

    def run(self):
        output = shellout("""python {script} <(curl --fail -sL {link}) {output}""",
                          link=self.link,
                          script=self.assets("129/129_marcbinary.py"))
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext="fincmarc.mrc"))



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301

# Copyright 2018 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
Helper for sending mail.
"""

import logging
import smtplib
import socket

from siskin.configuration import Config

config = Config.instance()
logger = logging.getLogger("siskin")

DEFAULT_SUBJECT_PREFIX = "[siskin at %s]" % (socket.gethostname())


def send_mail(sender=None, tolist=None, subject=None, message=None, smtp=None):
    """
    Send out an email. Configure `smtp`, `default-sender` in `core` config
    section. A subject prefix is always prepended. The `tolist` parameter can
    be a single string or a list of strings.

    XXX: ATM no FROM: line, maybe add that.
    """
    if subject is None:
        subject = "[automessage]"
    if smtp is None:
        smtp = config.get("core", "smtp")
    if sender is None:
        sender = config.get("core", "default-sender")

    if not all((sender, tolist, subject, smtp, message)):
        raise ValueError("missing sender, recipients, subject, message or smtp: %s" % locals())

    if not isinstance(tolist, list):
        tolist = [tolist]

    server = smtplib.SMTP(smtp)
    msg = 'Subject: {} {}\n\n{}'.format(DEFAULT_SUBJECT_PREFIX, subject, message)
    server.sendmail(sender, tolist, msg)
    server.quit()

    logger.debug("sent mail (%d) to %s -- %s", len(message), ", ".join(tolist), subject)



#!/usr/bin/env python
# coding: utf-8
#
# Copyright 2019 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""

Automated content verification of all sources in the (live) solr
Ticket: #15656

Usage:

    $ solrcheckup.py -d my.db -f 1.2.3.4:8080 -a solr.index.xyz --smtp-sender "beep@friendlyalarms.com"

This script should run fine in cron.
"""

import argparse
import io
import logging
import os
import re
import smtplib
import sqlite3
import sys
import tempfile
import time
from six.moves.urllib.parse import urlencode
from sqlite3 import Error
from siskin.mail import send_mail

import requests

logging.basicConfig(level=logging.DEBUG)

# The current database schema.
create_schema = """
    CREATE TABLE
        source
            (source INT PRIMARY KEY NOT NULL);

    CREATE TABLE
        institution
            (institution VARCHAR(30) PRIMARY KEY NOT NULL);

    CREATE TABLE
        sourcebyinstitution
            (sourcebyinstitution VARCHAR(30) PRIMARY KEY NOT NULL);

    CREATE TABLE
        history
            (date DEFAULT CURRENT_TIMESTAMP,
            sourcebyinstitution VARCHAR(30) NOT NULL,
            titles INT NOT NULL);
"""

# XXX: Encapsulate this better, to get rid of globals.
smtp_server = "mail.example.com" # XXX: use generic config for this
smtp_port = 465 # XXX: use generic config for this
smtp_name = "username" # XXX: unused?
smtp_password = "password" # XXX: unused?

smtp_sender = "noreply@example.com"
recipients = ["a@example.com", "b@example.com"]


def send_message(message):
    """
    Send e-mail to preconfigured recipients.
    """
    if not recipients:
        logging.warn("no recipients set, not sending any message")
        return

    send_mail(sender=smtp_sender,
              tolist=recipients,
              subject="SolrCheckup Warnung!",
              message=message)


def create_connection_and_set_cursor(database):
    """
    Creates a database connection to a SQLite database and returns a cursor
    """
    try:
        conn = sqlite3.connect(database)
    except Error as e:
        logging.error(e)
        sys.exit("No database connection could be established.")
    cursor = conn.cursor()
    return (conn, cursor)


def get_solr_result(index, params):
    """
    Takes a Solr index and a dict of parameters and returns a result object.
    Index should be hostport or ip:port, like 10.1.1.1:8085.
    """
    params = urlencode(params)
    result = requests.get("http://%s/solr/biblio/select?%s" % (index, params))
    return result.json()


def get_all_current_sources(finc, ai):
    """
    Get all current sources from Solr in both finc main index and ai.
    """
    params = {
        "facet": "true",
        "facet.field": "source_id",
        "facet.mincount": 3, # because of these cases ["", 2, "\" \"", 1]
        "q": "!source_id:error",
        "rows": 0,
        "wt": "json",
    }

    result = get_solr_result(finc, params)
    finc_sources = result["facet_counts"]["facet_fields"]["source_id"]
    finc_sources = set([int(sid) for sid in finc_sources[::2]])

    result = get_solr_result(ai, params)
    ai_sources = result["facet_counts"]["facet_fields"]["source_id"]
    ai_sources = set([int(sid) for sid in ai_sources[::2]])

    shared = finc_sources.intersection(ai_sources)
    if len(shared) > 0:
        ssid = [str(sid) for sid in shared]
        message = "Die folgenden Quellen befinden sich sowohl im finc-main-Index als auch im AI: {}".format(", ".join(ssid))
        send_message(message)

    return finc_sources.union(ai_sources)


def get_all_old_sources(conn, sqlite):
    """
    Get all old sources from the Database.
    """
    query = """
        SELECT
            source
        FROM
            source
        GROUP BY
            source
    """

    sqlite.execute(query)
    old_sources = []

    for record in sqlite:
        old_source = record[0]
        old_sources.append(old_source)

    return old_sources


def update_sources(conn, sqlite, finc, k10plus, ai):
    """
    Update the source table.
    """
    current_sources = get_all_current_sources(finc, ai)
    old_sources = get_all_old_sources(conn, sqlite)

    # Check if the source table is allready filled and this is not the first checkup
    source_table_is_filled = len(old_sources) > 100

    for old_source in old_sources:
        if source_table_is_filled and old_source not in current_sources:
            message = "Die SID %s ist im aktuellen Import nicht mehr vorhanden.\nWenn dies beabsichtigt ist, bitte die SID aus der Datenbank loeschen." % old_source
            send_message(message)

    for current_source in current_sources:
        if current_source not in old_sources:
            logging.info("The source %s is new in Solr.", current_source)
            sql = "INSERT INTO source (source) VALUES (%s)" % current_source
            sqlite.execute(sql)
            conn.commit()


def get_all_current_institutions(finc, ai):
    """
    Get all current institutions from Solr.
    """
    current_institutions = []

    params = {
        "facet": "true",
        "facet.field": "institution",
        "facet.mincount": 3, # because of these cases ["", 2, "\" \"", 1]
        "q": "!source_id:error",
        "rows": 0,
        "wt": "json",
    }

    # check finc main index
    result = get_solr_result(finc, params)
    institutions = result["facet_counts"]["facet_fields"]["institution"]
    for institution in institutions[::2]:
        current_institutions.append(institution)

    # check ai
    result = get_solr_result(ai, params)
    institutions = result["facet_counts"]["facet_fields"]["institution"]
    for institution in institutions[::2]:
        if institution in current_institutions:
            continue
        current_institutions.append(institution)

    return current_institutions


def get_all_old_institutions(conn, sqlite):
    """
    Get all old institutions from the SQLite database.
    """
    query = """
        SELECT
            institution
        FROM
            institution
        GROUP BY
            institution
    """

    sqlite.execute(query)
    old_institutions = []

    for record in sqlite:
        old_institution = record[0]
        old_institutions.append(old_institution)

    return old_institutions


def get_all_old_sourcebyinstitutions(conn, sqlite):
    """
    Get all old sourcebyinstitution from the SQLite database.
    """
    query = """
        SELECT
            sourcebyinstitution
        FROM
            sourcebyinstitution
        GROUP BY
            sourcebyinstitution
    """

    sqlite.execute(query)
    old_sourcebyinstitutions = []

    for record in sqlite:
        old_sourcebyinstitution = record[0]
        old_sourcebyinstitutions.append(old_sourcebyinstitution)

    return old_sourcebyinstitutions


def get_old_sourcebyinstitution_number(conn, sqlite, sourcebyinstitution):
    """
    Get all the old sourcebyinstitution number from the SQLite database.
    """
    query = """
        SELECT
            titles
        FROM
            history
        WHERE
            sourcebyinstitution = "%s"
        ORDER BY
            titles DESC
        LIMIT 1
    """ % sourcebyinstitution

    sqlite.execute(query)
    for record in sqlite:
        old_sourcebyinstitution_number = record[0]
        return old_sourcebyinstitution_number


def update_institutions(conn, sqlite, finc, k10plus, ai):
    """
    Update the institution table.
    """
    current_institutions = get_all_current_institutions(finc, ai)
    old_institutions = get_all_old_institutions(conn, sqlite)

    # Check if the institution table is allready filled and this is not the first checkup
    institution_table_is_filled = len(old_institutions) > 5

    for old_institution in old_institutions:
        if institution_table_is_filled and old_institution not in current_institutions:
            message = "Die ISIL %s ist im aktuellen Import nicht mehr vorhanden.\nWenn dies beabsichtigt ist, bitte die Institution aus der Datenbank loeschen." % old_institution
            send_message(message)

    for current_institution in current_institutions:
        if current_institution == " " or '"' in current_institution:
                continue
        if current_institution not in old_institutions:
            logging.info("The institution %s is new in Solr.", current_institution)
            sql = "INSERT INTO institution (institution) VALUES ('%s')" % current_institution
            sqlite.execute(sql)
            conn.commit()


def update_history_and_sourcebyinstitution(conn, sqlite, finc, k10plus, ai):
    """
    Get all current sources and title numbers from Solr and log them into database.
    """
    current_sources = get_all_current_sources(finc, ai)
    current_institutions = get_all_current_institutions(finc, ai)
    old_sourcebyinstitutions = get_all_old_sourcebyinstitutions(conn, sqlite)
    current_sourcebyinstitutions = []

    for source in current_sources:

        for institution in current_institutions:

            if not institution or institution == " " or '"' in institution:
                continue

            sourcebyinstitution = "SID " + str(source) + " (" + institution + ")"
            current_sourcebyinstitutions.append(sourcebyinstitution)

            params = {
                "q": 'source_id:%s AND institution:"%s"' % (source, institution),
                "rows": 0,
                "wt": "json"
            }

            # check finc main
            result = get_solr_result(finc, params)
            number = result["response"]["numFound"]
            if number != 0:
                sql = 'INSERT INTO history (sourcebyinstitution, titles) VALUES ("%s", %s)' % (sourcebyinstitution, number)
                sqlite.execute(sql)
                conn.commit()
            else:
                # check ai
                result = get_solr_result(ai, params)
                number = result["response"]["numFound"]
                if number != 0:
                    # TODO: escape via sqlite
                    sql = 'INSERT INTO history (sourcebyinstitution, titles) VALUES ("%s", %s)' % (sourcebyinstitution, number)
                    sqlite.execute(sql)
                    conn.commit()

            if sourcebyinstitution not in old_sourcebyinstitutions:
                logging.info("The %s is now connected to SID %s.", institution, source)
                sql = "INSERT INTO sourcebyinstitution (sourcebyinstitution) VALUES ('%s')" % sourcebyinstitution
                sqlite.execute(sql)
                conn.commit()

            if number != 0:
                old_sourcebyinstitution_number = get_old_sourcebyinstitution_number(conn, sqlite, sourcebyinstitution)
                if number < old_sourcebyinstitution_number:
                    message = "Die Anzahl der Titel hat sich bei %s gegenueber einem frueheren Import verringert." % (sourcebyinstitution)
                    send_message(message)

            # requests.exceptions.ConnectionError: HTTPConnectionPool(XXXXXX): Max retries exceeded
            time.sleep(0.25)

    for old_sourcebyinstitution in old_sourcebyinstitutions:
        if old_sourcebyinstitution not in current_sourcebyinstitutions:
            message = "Die %s ist nicht laenger für die SID %s angesigelt." % (institution, source)
            send_message(message)

# Parse keyword arguments
parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
parser.add_argument("-v",
                    action="version",
                    help="show version",
                    version="0.0.1")
parser.add_argument("-d",
                    dest="database",
                    help="path to database",
                    default=os.path.join(tempfile.gettempdir(), "solrcheckup.sqlite"),
                    metavar="database")
parser.add_argument("-y",
                    dest="yaml",
                    help="link to review.yaml or another yaml template",
                    metavar="yaml")
parser.add_argument("-t",
                    dest="token",
                    help="private token for GitLab",
                    metavar="token")
parser.add_argument("-f",
                    dest="finc",
                    help="url of the finc main index",
                    metavar="finc",
                    required=True)
parser.add_argument("-k",
                    dest="k10plus",
                    help="url of the k10plus index",
                    metavar="k10plus")
parser.add_argument("-a",
                    dest="ai",
                    help="url of the ai index",
                    metavar="ai",
                    required=True)
parser.add_argument("-n", "--smtp-name",
                    dest="smtp_name",
                    help="the login name fpr the email account",
                    metavar="smtp-name")
parser.add_argument("-p", "--smtp-password",
                    dest="smtp_password",
                    help="the password of the email account",
                    metavar="smtp_password")
parser.add_argument("--smtp-server",
                    dest="smtp_server",
                    help="SMTP server",
                    metavar="smtp_server")
parser.add_argument("--smtp-port",
                    dest="smtp_port",
                    help="SMTP port",
                    metavar="smtp_port",
                    default=465)
parser.add_argument("--smtp-sender",
                    dest="smtp_sender",
                    help="SMTP from address",
                    metavar="smtp_sender",
                    default="noreply@example.com")
parser.add_argument("--recipients",
                    dest="recipients",
                    help="recipients for alert messages, comma separated",
                    default="",
                    metavar="recipients")

args = parser.parse_args()

# XXX: Reduce use of globals.
smtp_server = args.smtp_server
smtp_port = args.smtp_port
smtp_server = args.smtp_server
smtp_sender = args.smtp_sender
smtp_name = args.smtp_name
smtp_password = args.smtp_password
recipients = [addr.strip() for addr in args.recipients.split(",") if addr.strip()]

database = args.database

# Exit when using yaml template without private token
yaml = args.yaml
token = args.token
if yaml and not token:
    sys.exit("Keyword argument for private token needed when using yaml template.")

# Ensure that all three indicies are specified
finc = args.finc
k10plus = args.k10plus
ai = args.ai


# Check if database already exists, otherwise create new one
if not os.path.isfile(database):
    conn, sqlite = create_connection_and_set_cursor(database)
    sqlite.executescript(create_schema)
else:
    conn, sqlite = create_connection_and_set_cursor(database)

# 1. Step: Update the source table
update_sources(conn, sqlite, finc, k10plus, ai)

# 2. Step: Update the institution table
update_institutions(conn, sqlite, finc, k10plus, ai)

# 3. Step: Get the number of titles for each SID and log them to database
update_history_and_sourcebyinstitution(conn, sqlite, finc, k10plus, ai)

sqlite.close()
